/**
 * 
 */
package org.gcube.accounting.datamodel.usagerecords;

import java.util.HashSet;
import java.util.Set;

import org.gcube.accounting.datamodel.BasicUsageRecord;
import org.gcube.accounting.datamodel.BasicUsageRecordUtility;
import org.gcube.accounting.datamodel.basetypes.AbstractJobUsageRecord;
import org.gcube.accounting.datamodel.basetypes.TestUsageRecord;
import org.gcube.accounting.exception.InvalidValueException;
import org.gcube.common.scope.api.ScopeProvider;
import org.junit.Assert;
import org.junit.Test;

/**
 * @author Luca Frosini (ISTI - CNR) http://www.lucafrosini.com/
 *
 */
public class JobUsageRecordTest {

	public static Set<String> getExpectedRequiredFields(){
		Set<String> expectedRequiredFields = new HashSet<String>();
		expectedRequiredFields.add(BasicUsageRecordUtility.ID);
		expectedRequiredFields.add(BasicUsageRecord.CONSUMER_ID);
		expectedRequiredFields.add(BasicUsageRecord.CREATION_TIME);
		expectedRequiredFields.add(BasicUsageRecordUtility.USAGE_RECORD_TYPE);
		expectedRequiredFields.add(BasicUsageRecord.SCOPE);
		expectedRequiredFields.add(BasicUsageRecord.OPERATION_RESULT);
		expectedRequiredFields.add(AbstractJobUsageRecord.JOB_ID);
		expectedRequiredFields.add(AbstractJobUsageRecord.JOB_QUALIFIER);
		expectedRequiredFields.add(AbstractJobUsageRecord.JOB_NAME);
		expectedRequiredFields.add(AbstractJobUsageRecord.JOB_START_TIME);
		expectedRequiredFields.add(AbstractJobUsageRecord.JOB_END_TIME);
		return expectedRequiredFields;
	}
	
	@Test(expected=InvalidValueException.class)
	public void scopeNotSetValidationError() throws InvalidValueException {
		ScopeProvider.instance.reset();
		JobUsageRecord jobUsageRecord = TestUsageRecord.createTestJobUsageRecordAutomaticScope();
		jobUsageRecord.validate();
	}
	
	@Test
	public void testRequiredFields() throws InvalidValueException{
		ScopeProvider.instance.set(TestUsageRecord.TEST_SCOPE);
		JobUsageRecord jobUsageRecord = TestUsageRecord.createTestJobUsageRecordAutomaticScope();
		Set<String> expectedRequiredFields = getExpectedRequiredFields();
		Set<String> gotRequiredFields = jobUsageRecord.getRequiredFields();

		Assert.assertTrue(expectedRequiredFields.containsAll(gotRequiredFields));
		Assert.assertTrue(gotRequiredFields.containsAll(expectedRequiredFields));
		
		jobUsageRecord.validate();
	}
	
	@Test
	public void testRequiredFieldsWithScopeSetExplicitely() throws InvalidValueException{
		JobUsageRecord jobUsageRecord = TestUsageRecord.createTestJobUsageRecordAutomaticScope();
		jobUsageRecord.setScope(TestUsageRecord.TEST_SCOPE);
		Set<String> expectedRequiredFields = getExpectedRequiredFields();
		
		Set<String> gotRequiredFields = jobUsageRecord.getRequiredFields();

		Assert.assertTrue(expectedRequiredFields.containsAll(gotRequiredFields));
		Assert.assertTrue(gotRequiredFields.containsAll(expectedRequiredFields));
		
		jobUsageRecord.validate();
	}
}
