/**
 * 
 */
package org.gcube.accounting.datamodel.usagerecords;

import java.util.HashSet;
import java.util.Set;

import org.gcube.accounting.datamodel.BasicUsageRecord;
import org.gcube.accounting.datamodel.BasicUsageRecordUtility;
import org.gcube.accounting.datamodel.basetypes.TestUsageRecord;
import org.gcube.accounting.exception.InvalidValueException;
import org.gcube.common.scope.api.ScopeProvider;
import org.junit.Assert;
import org.junit.Test;

/**
 * @author Luca Frosini (ISTI - CNR) http://www.lucafrosini.com/
 *
 */
public class ServiceUsageRecordTest {

	public static Set<String> getExpectedRequiredFields(){
		Set<String> expectedRequiredFields = new HashSet<String>();
		expectedRequiredFields.add(BasicUsageRecordUtility.ID);
		expectedRequiredFields.add(BasicUsageRecord.CONSUMER_ID);
		expectedRequiredFields.add(BasicUsageRecord.CREATION_TIME);
		expectedRequiredFields.add(BasicUsageRecordUtility.USAGE_RECORD_TYPE);
		expectedRequiredFields.add(BasicUsageRecord.SCOPE);
		expectedRequiredFields.add(BasicUsageRecord.OPERATION_RESULT);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.CALLER_HOST);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.HOST);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.SERVICE_CLASS);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.SERVICE_NAME);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.CALLED_METHOD);
		expectedRequiredFields.add(org.gcube.accounting.datamodel.basetypes.AbstractServiceUsageRecord.DURATION);
		return expectedRequiredFields;
	}
	
	@Test(expected=InvalidValueException.class)
	public void scopeNotSetValidationError() throws InvalidValueException {
		ScopeProvider.instance.reset();
		ServiceUsageRecord serviceUsageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
		serviceUsageRecord.validate();
	}
	
	@Test
	public void testRequiredFields() throws InvalidValueException{
		ScopeProvider.instance.set(TestUsageRecord.TEST_SCOPE);
		ServiceUsageRecord serviceUsageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
		Set<String> expectedRequiredFields = getExpectedRequiredFields();
		Set<String> gotRequiredFields = serviceUsageRecord.getRequiredFields();

		Assert.assertTrue(expectedRequiredFields.containsAll(gotRequiredFields));
		Assert.assertTrue(gotRequiredFields.containsAll(expectedRequiredFields));
		
		serviceUsageRecord.validate();
	}
	
	@Test
	public void testRequiredFieldsWithScopeSetExplicitely() throws InvalidValueException{
		ServiceUsageRecord serviceUsageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
		serviceUsageRecord.setScope(TestUsageRecord.TEST_SCOPE);
		Set<String> expectedRequiredFields = getExpectedRequiredFields();
		
		Set<String> gotRequiredFields = serviceUsageRecord.getRequiredFields();

		Assert.assertTrue(expectedRequiredFields.containsAll(gotRequiredFields));
		Assert.assertTrue(gotRequiredFields.containsAll(expectedRequiredFields));
		
		serviceUsageRecord.validate();
	}
}
