/**
 * 
 */
package org.gcube.accounting.persistence;

import java.util.concurrent.TimeUnit;

import org.gcube.accounting.datamodel.SingleUsageRecord;
import org.gcube.accounting.datamodel.basetypes.TestUsageRecord;
import org.gcube.accounting.exception.InvalidValueException;
import org.gcube.accounting.testutility.StressTestUtility;
import org.gcube.accounting.testutility.TestOperation;
import org.gcube.common.scope.api.ScopeProvider;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * @author Luca Frosini (ISTI - CNR) http://www.lucafrosini.com/
 *
 */
public class AccountingPersistenceBackendMonitorTest {
	
	private static final Logger logger = LoggerFactory.getLogger(AccountingPersistenceBackendMonitor.class);
	
	public static final long timeout = 5000;
	public static final TimeUnit timeUnit = TimeUnit.MILLISECONDS;
	
	@Test
	public void parsingTest() throws Exception {
		ScopeProvider.instance.set("/gcube/devsec");
		
		AccountingPersistenceBackendFactory.setFallbackLocation(null);
		final AccountingPersistence persistence = AccountingPersistence.getInstance();
		
		StressTestUtility.stressTest(new TestOperation() {
			
			@Override
			public void operate(int i) {
				SingleUsageRecord usageRecord = null;
				switch (i%3) {
					case 0:
						usageRecord = TestUsageRecord.createTestServiceUsageRecordAutomaticScope();
						break;
					case 1:
						usageRecord = TestUsageRecord.createTestStorageUsageRecordAutomaticScope();
						break;
					case 2:
						usageRecord = TestUsageRecord.createTestJobUsageRecordAutomaticScope();
						break;
				}
				try {
					persistence.account(usageRecord);
				} catch (InvalidValueException e) {
					throw new RuntimeException(e);
				}
			}
		});
		
		logger.debug(" START -----------------------------------------------");
		logger.debug("Flushing the buffered records");
		persistence.flush(timeout, timeUnit);
		logger.debug(" END -----------------------------------------------");
		
		AccountingPersistenceBackend accountingPersistenceBackend = AccountingPersistenceBackendFactory.getPersistenceBackend();
		accountingPersistenceBackend.setFallback((FallbackPersistenceBackend) accountingPersistenceBackend);
		AccountingPersistenceBackendMonitor accountingPersistenceBackendMonitor = new AccountingPersistenceBackendMonitor(accountingPersistenceBackend); 
		
		accountingPersistenceBackendMonitor.run();
		
	}
	
	@Test
	public void singleParsingTest() throws Exception {
		ScopeProvider.instance.set("/gcube/devsec");
		AccountingPersistenceBackendFactory.setFallbackLocation(null);
		AccountingPersistenceBackend accountingPersistenceBackend = AccountingPersistenceBackendFactory.getPersistenceBackend();
		accountingPersistenceBackend.setFallback((FallbackPersistenceBackend) accountingPersistenceBackend);
		AccountingPersistenceBackendMonitor accountingPersistenceBackendMonitor = new AccountingPersistenceBackendMonitor(accountingPersistenceBackend); 
		accountingPersistenceBackendMonitor.run();
	}
	
}
